#include "app.h"
#include "background.h"

UINT32				state, item_select;							
static UINT16       status;
static WCHAR        file_cfg[256];			// Jabber.cfg
WCHAR				icons_dir[128];

AHIBITMAP_T			cusor_bitmap;
LIST 				ditems;
UIS_DIALOG_T        dialog; 				// Диалог
ldrElf       	 	elf;					// Структура эльфа
DRAWING_BUFFER_T 	bufd = {NULL, 176, 220};// Буфер для рисования, Canvas.
							
/* Название приложения. Длина строки именно такая и никакая иначе */
const char app_name[APP_NAME_LEN] = ELF_NAME; 

/* Обработчики событий для каждого state-а */
/* Обработчики событий для APP_STATE_ANY (используется в любом state) */
static EVENT_HANDLER_ENTRY_T any_state_handlers[] =
{
	{ EV_PM_API_EXIT,				 HandleReqExit },	// PM API: cmd EXIT
	{ STATE_HANDLERS_RESERVED,	 HandleChangeState },
    { STATE_HANDLERS_END,           		  NULL },
};

/* Таблица соответствий обработчиков, состояний и функций входа-выхода из состояния.
   Порядок состояний такой же, как в enum-e */
static const STATE_HANDLERS_ENTRY_T state_handling_table[] =
{
    { APP_STATE_ANY,              // State
      NULL,                       // Обработчик входа в state
      NULL,                       // Обработчик выхода из state
      any_state_handlers          // Список обработчиков событий
    },
	{ APP_STATE_BG,
      BackgroundStateEnter,
	  BackgroundStateExit,
      background_state_handlers
    },
    { APP_STATE_MAIN,
      MainStateEnter,
      StateExit,
      NULL
    },
	{ APP_STATE_EDIT,
      EditStateEnter,
      StateExit,
      NULL
	}
	
};

static const APP_STATE_NAME_TABLE_T state_name_table[] =
{
	{ APP_STATE_ANY,			"ANY" },
	{ APP_STATE_BG,		 "BACKGROUND" },
	{ APP_STATE_MAIN,		   "MAIN" },
	{ APP_STATE_EDIT,		   "EDIT" },
	{ APP_STATE_MAX,			 NULL }
};

/* Это entry для всех эльфов, должна быть объявлена именно так */
/* uri - путь к эльфу (аналогично argv[0]) */
/* params - параметры эльфа (аналогично остальным argv) */
ldrElf * _start( WCHAR * uri, WCHAR * params )
{
    ldrElf *	pelf;
	UINT32		reserve;
	int			i;
	
	dbg("-----------------------------------");
	dbg("_start: Enter");
	
	// ищем запущеный эльф по имени эльфа
	pelf = ldrFindElf(app_name);
	if ( pelf != NULL ) {
		// приложение уже запущено, вызвать запущеное
		ldrSendEvent( pelf->evbase );
		return NULL;
	}
	// база ивентов
	reserve = ldrRequestEventBase();
	// заполняем структуру эльфа
	elf.name = (char *)app_name;
	elf.app = NULL;
	
	// инитим таблицы ивентов
	for ( i=0; i<APP_STATE_MAX; i++ )
		reserve = ldrInitEventHandlersTbl((EVENT_HANDLER_ENTRY_T *)state_handling_table[i].htable, reserve);
	
	// папка эльфа
	i = u_strlen( uri ) - 1;
	while ( i >= 0 && uri[i] != L'/' ) i--;
	i++;
	elf.dir = malloc( i * sizeof(WCHAR) );
	if ( elf.dir )
		u_strncpy( elf.dir, uri, i );
	
	// получаем путь к конфигу	
	u_mkpath(file_cfg, elf.dir, L"toolbar.cfg", 0);
	
	elf.evbase = reserve ++;
	
	/* Регистрируем приложение */
	 status = APP_Register( &elf.evbase,            // На какое событие должно запускаться приложение
                           1,                       // Кол-во событий, на которое должно запускаться приложение
                           state_handling_table,    // Таблица состояний
                           APP_STATE_MAX,            // Количество состояний
                           (void*)startApp );   // Функция, вызываемая при запуске
	
	if ( status == RESULT_OK )
		ldrSendEvent( elf.evbase );
	 else {
		cprint(ELF_NAME ":\x84 can't register app\n");
		return NULL;
	}

#ifdef DEBUG
	ReportPhoneInfo();
#endif

    return &elf; 
}

/* Функция вызываемая при старте приложения */
UINT32 startApp( EVENT_STACK_T *ev_st,  REG_ID_T reg_id,  UINT32 param2 )
{
	APP_ELF_T *			papp;
	
	// ... в папке скина
	u_mkpath(icons_dir, SKIN_FOLDER, UIS_GetSkinName( UIS_GetCurrentSkinIndex() ), 0);
	u_strcat( icons_dir, L"/icons/" );
	
	if( !DL_FsDirExist(icons_dir) ) 
		u_mkpath(icons_dir, elf.dir, L"icons/", 0);
	
	if( !DL_FsDirExist(icons_dir) ) {
		cprint(ELF_NAME ": can't init resouces\n");
		ldrUnloadElf();
		return RESULT_FAIL;
	}
	
	if( AFW_InquireRoutingStackByRegId( reg_id ) == RESULT_OK ) {
		appShow( elf.app );
		return RESULT_FAIL;
	}
	
    papp = (APP_ELF_T*)APP_InitAppData((void*)AppHandleEvent, 
                                            sizeof(APP_ELF_T), // Размер структуры приложения
                                            reg_id,
                                            0,
											0,			// history size
											1,			// priority
											AFW_APP_CENTRICITY_NONE,
											AFW_PREPROCESSING,
											AFW_POSITION_TOP );	
	
	if ( !papp )
	{
		dbg("can't init app data");
		ldrUnloadElf();
		return RESULT_FAIL;
	}
	state = MAIN;
	List_Init( &ditems, DESKTOP_ITEM );
	ReadSetting();
	InitResources();	// инит скина
    status = APP_Start( ev_st,
                        &papp->apt,
                        APP_STATE_BG, // Начальное состояние
                        state_handling_table,
                        destroyApp,
                        app_name,
                        state_name_table );
	
	// Теперь приложение может обрабатывать события
	if ( status == RESULT_OK ) {
		elf.app = &papp->apt;
		cprint(ELF_NAME ": Started\n");
	}
    return RESULT_OK;
}


/* Функция выхода из приложения */
UINT32 destroyApp( EVENT_STACK_T *ev_st,  APPLICATION_T *app )
{
	EVENT_T *		event = AFW_GetEv(ev_st);
	dbgf("destroyApp: Enter, event = 0x%p, code = 0x%X", event, event->code);
	
	if ( event != NULL ) {
		// это нажатие красной кнопки
		appHide(app);
		return RESULT_OK;
	}
		
	cprint( ELF_NAME ":\x8E Exit\n" );
	
	SaveSetting();
	FreeResources();
	List_Clear(&ditems);

    /* Завершаем работу приложения */
    status = APP_ExitStateAndApp( ev_st, app, 0 );
	
	ldrUnloadElf();
   
    return status;
}
// Изменить стэйт + насильно показать
UINT32 AppChangeState( APPLICATION_T * app, UINT8 new_state, BOOL force_show )
{
	ADD_EVENT_DATA_T	ed;
	UINT32				ev_code;
	
	ed.data_tag = 0;
	ed.data.pad[0] = new_state;
	ed.data.pad[1] = force_show;
	
	ev_code = ldrFindEventHandlerTbl(any_state_handlers, HandleChangeState);
	if ( ev_code > 0 )
		return
		AFW_CreateInternalQueuedEvPriv( ev_code, 0,
										app->afw_id,
										0, 0,
										&ed,
										FBF_FREE, 0, NULL, 0 );
	else
		return RESULT_FAIL;
}

UINT32 HandleChangeState( EVENT_STACK_T * ev_st, APPLICATION_T * app )
{
	APP_ELF_T		*papp = (APP_ELF_T *)(app);
	UINT32			status;
	EVENT_T *		event;
	
	status = RESULT_OK;
	event = AFW_GetEv(ev_st);
	APP_ConsumeEv(ev_st, app);
	if ( app->state == APP_STATE_BG && event->data.pad[0] != APP_STATE_BG ) {
		// если уже в бэке
		papp->last_state = event->data.pad[0];
		if ( event->data.pad[1] == TRUE ) {
			// нужно вызвать из бэка
			status = APP_UtilChangeState(papp->last_state, ev_st, app);
		}
	}
	else if ( event->data.pad[0] == APP_STATE_BG ) {
		// отправить в бэк
		if ( app->state != APP_STATE_BG ) {
			papp->last_state = app->state;
			status = APP_UtilChangeState(APP_STATE_BG, ev_st, app);
		}
	}
	else {
		// просто изменить стэйт
		status = APP_UtilChangeState(event->data.pad[0], ev_st, app);
	}
	return status;
}

/* Обработчик входа в main state */
UINT32 MainStateEnter( EVENT_STACK_T *ev_st,  APPLICATION_T *app,  ENTER_STATE_TYPE_T type )
{
	UIX_CALL_BACK_F 		*CallBackArr[2];
	UINT32					num;	// число пунктов в списке
	
	dbg("MainStateEnter: start");
	if(type != ENTER_STATE_ENTER) return UIX_Render( app->dialog );
	
	CallBackArr[0] = (void *)HandleSelect;
	CallBackArr[1] = (void *)HandleBack;
	CallBackArr[2] = (void *)HandleNavigate;
	
	switch (state)
	{
		case MAIN:
			num = List_GetNum(&ditems);
			break;
			
		case EDIT_ITEM:
			num = 5;
			break;
		
	}
	
	app->dialog = UIX_CreateListDialog(app, num, MainEntryReq, CallBackArr);
	
	//UIX_DialogAddAction(app->dialog, "Hide", (void *)HandleHide, ACTION_FLAG_LEFT_SOFT );
		
	UIX_DialogAddAction(app->dialog, "New item", ( void *)HandleNew, 0 );
	UIX_DialogAddAction(app->dialog, "Delete item", ( void *)HandleDelete, 0 );
	UIX_DialogAddAction(app->dialog, "Hide", (void *)HandleHide, 0);
	UIX_DialogAddAction(app->dialog, "Exit", ( void *)HandleExit, 0 );
		
	UIX_DialogSetTitle(app->dialog, ELF_NAME " " ELF_VER "by R.U.S.H.A.N.", ElfGetIcon(0));
	
	UIX_Render(app->dialog);
	
	dbg("MainStateEnter: end");
	
    return RESULT_OK;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 EditStateEnter( EVENT_STACK_T *ev_st,  APPLICATION_T *app,  ENTER_STATE_TYPE_T type )
{
	dbgf("EditStateEnter: start, type = %d, substate = %d", type, state );
	if(type != ENTER_STATE_ENTER) return UIX_Render( app->dialog );
	
	char					buf[8];
	
	switch(state)
	{
		case EDIT_ITEM_X:
		app->dialog = UIX_CreateInputDialog(app, 3, itoa(ElfGetDItem(item_select)->x, buf, 10), INP_USE_DIGITAL, ( void *)EditDone );
		UIX_DialogSetTitle(app->dialog, "Enter item X:", ElfGetIcon(0));
        break;

		case EDIT_ITEM_Y:
		app->dialog = UIX_CreateInputDialog(app, 3, itoa(ElfGetDItem(item_select)->y, buf, 10), INP_USE_DIGITAL, ( void *)EditDone );
		UIX_DialogSetTitle(app->dialog, "Enter item Y:", ElfGetIcon(0));
        break;
		
		case EDIT_ITEM_NAME:
		app->dialog = UIX_CreateInputDialog(app, 16, ElfGetDItem(item_select)->name, INP_USE_DIGITAL | INP_USE_ENGLISH | INP_USE_OTHERS, ( void *)EditDone );
        UIX_DialogSetTitle(app->dialog, "Enter item name:", ElfGetIcon(0));
		break;
		
		case EDIT_ITEM_ELF_PATH:
		app->dialog = UIX_CreateInputDialog(app, 128, ElfGetDItem(item_select)->elf_path, INP_USE_DIGITAL | INP_USE_ENGLISH | INP_USE_OTHERS, ( void *)EditDone );
        UIX_DialogSetTitle(app->dialog, "Enter item elf path:", ElfGetIcon(0));
		break;
		
		case EDIT_ITEM_EVENT:
		app->dialog = UIX_CreateInputDialog(app, 4, itoa(ElfGetDItem(item_select)->event, buf, 10), INP_USE_DIGITAL, ( void *)EditDone );
		UIX_DialogSetTitle(app->dialog, "Enter item event:", ElfGetIcon(0));
        break;
	} 

	return UIX_Render(app->dialog);
}

UINT32 StateExit( EVENT_STACK_T *ev_st,  APPLICATION_T *app,  EXIT_STATE_TYPE_T type )
{
	if ( type != EXIT_STATE_EXIT )
		return RESULT_OK;
	
	status = APP_UtilUIXDialogDelete( &app->dialog );

	return status;
}
UINT32 HandleReqExit( EVENT_STACK_T *ev_st,  APPLICATION_T *app )
{
	dbg("HandleReqExit");
	APP_ConsumeEv( ev_st, app );
	app->exit_status = true;

	return RESULT_OK;
}
UINT32 HandleNavigate( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 idx )
{
	if ( state == MAIN ) 
		item_select = idx;	// индекс выбранного пункта
		
	return RESULT_OK;
}
// выбрать
UINT32 HandleSelect( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 idx )
{	
	
	dbg("HandleSelect");
	APP_ConsumeEv( ev_st, app );
	
	switch (state)
	{	
		case MAIN:
			state = EDIT_ITEM;
			item_select = idx;
			return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );
			
		case EDIT_ITEM:
			if( idx == 3 ) {
				if ( ElfGetDItem(item_select)->dtype ) 
					ElfGetDItem(item_select)->dtype = false;
				else 
					ElfGetDItem(item_select)->dtype = true;
				return UIX_Render(app->dialog);  		
			}
			if( idx == 4 ) {
				if ( ElfGetDItem(item_select)->dtype == DE_DATA_ELF ) 
					state = EDIT_ITEM_ELF_PATH;
				else
					state = EDIT_ITEM_EVENT;
			} else
				state = EDIT_ITEM_X + idx;
			return APP_UtilChangeState( APP_STATE_EDIT, ev_st, app );
	}
	
	return RESULT_OK;
}
// назад
UINT32 HandleBack( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 param )
{
	APP_ConsumeEv( ev_st, app );


	switch (state)
	{
		case MAIN:
			return appHide( app );
			
		case EDIT_ITEM:
			state = MAIN;
			return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );
		
	}
	
	return RESULT_OK;
}
// Новый пункт
UINT32 HandleNew( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 idx )
{
	APP_ELF_T			*papp = (APP_ELF_T*)app;
	DESKTOP_ITEM 		d;

	d.x = 2;
	d.y = 35;
	d.event = 0;
	
	d.dtype = DE_DATA_ELF;
	strcpy( d.name, "new" );	
	strcpy( d.elf_path, "/b/Elf2/.elf" );	

	List_AppendItem( &ditems, &d );
	
	state = MAIN;
	return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );
}

UINT32 HandleDelete( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 idx )
{
	APP_ELF_T			*papp = (APP_ELF_T*)app;

	List_DeleteItem( &ditems, item_select );
	
	state = MAIN;
	return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );
}
// Свернуть
UINT32 HandleHide( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 mode )
{	
	if ( mode == ACTION_KEY_RELEASE || mode == ACTION_LIST )
		return appHide(app);
	return RESULT_OK;
}
// Выход
UINT32 HandleExit( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 mode )
{
	if ( mode == ACTION_KEY_RELEASE || mode == ACTION_LIST )
		return HandleReqExit(ev_st, app);
	return RESULT_OK;
}

UINT32 EditDone( EVENT_STACK_T * ev_st, APPLICATION_T * app, UINT32 l  )
{
	char			*data;
	
	if ( l == (UINT32)(-1) || l == 0 ) {
		// Cancel
		switch(state)
		{
			case EDIT_ITEM_X:
			case EDIT_ITEM_Y:
			case EDIT_ITEM_NAME:
			case EDIT_ITEM_ELF_PATH:
			case EDIT_ITEM_EVENT:
				state = EDIT_ITEM;
				return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );
		}
	}
	
	data = malloc( sizeof(char)*(l+1) );
	
	UIX_InputGetData(app->dialog, data, 0);
	
		switch(state)
		{
			case EDIT_ITEM_X:
				state = EDIT_ITEM;
				ElfGetDItem(item_select)->x =  strtol(data, '\0', 10);
				break;
				
			case EDIT_ITEM_Y:
				state = EDIT_ITEM;
				ElfGetDItem(item_select)->y =  strtol(data, '\0', 10);
				break;
				
			case EDIT_ITEM_NAME:
				state = EDIT_ITEM;
				strcpy(ElfGetDItem(item_select)->name, data);
				break;
				
			case EDIT_ITEM_ELF_PATH:
				state = EDIT_ITEM;
				strcpy(ElfGetDItem(item_select)->elf_path, data);
				break;
				
			case EDIT_ITEM_EVENT:
				state = EDIT_ITEM;
				ElfGetDItem(item_select)->event =  strtol(data, '\0', 10);
				break;			
		}
		
	mfree(data);
	SaveSetting();
	
    return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );		
}

static void * MainEntryReq( APPLICATION_T * app, UINT32 idx )
{
	UIX_LIST_ENTRY_T    *e;
	UIX_ICON_T			icon;
	
	// динамически создаем
	e = suAllocMem(sizeof(UIX_LIST_ENTRY_T), NULL);
	memclr(e, sizeof(UIX_LIST_ENTRY_T));
	
	
	if ( state == MAIN ) {
	
		e->name = ElfGetDItem( idx )->name;
		
	} else {
	
		e->flags = LE_FLAG_HAS_DATA | LE_FLAG_FREE_MEM;
		
		switch ( idx )
		{
			case 0:
				e->name = "X:";
				e->dtype = LE_DATA_TYPE_INT32;
				e->d.snum = ElfGetDItem(item_select)->x;
			break;
			
			case 1:
				e->name = "Y:";
				e->dtype = LE_DATA_TYPE_INT32;
				e->d.snum = ElfGetDItem(item_select)->y;
			break;
			
			case 2:
				e->name = "Name:";
				e->dtype = LE_DATA_TYPE_STR;
				e->d.s = ElfGetDItem(item_select)->name;
			break;
			
			case 3:
				e->name = "Type:";
				e->dtype = LE_DATA_TYPE_STR;
				if( ElfGetDItem(item_select)->dtype == DE_DATA_ELF )
					e->d.s = "Elf";
				else 
					e->d.s = "Event";
			break;
			
			case 4:
				if( ElfGetDItem(item_select)->dtype == DE_DATA_ELF ) {
					e->name = "Elf path:";
					e->dtype = LE_DATA_TYPE_STR;
					e->d.s = ElfGetDItem(item_select)->elf_path;
				} else {
					e->name = "Event code:";
					e->dtype = LE_DATA_TYPE_UINT32;
					e->d.unum = ElfGetDItem(item_select)->event;
				}
			break;
			
		}
	
	}
	return e;
}

UINT32 ReadSetting() 
{
	FILE_HANDLE_T 		f;
	
	// читаем данные из конфига
	if (DL_FsFFileExist(file_cfg)) { // проверяем есть ли файл
	
		f = DL_FsOpenFile(file_cfg, FILE_READ_MODE, 0);
		
		List_ReadFile( &ditems, f );
		
        DL_FsCloseFile(f);
		
        dbg("ReadSetting: done");
    } else {	// если нет его, то грузим конфиг по умолчанию
		
		DESKTOP_ITEM 		d;

		d.x = 2;
		d.y = 35;
		d.event = 0;

		d.dtype = DE_DATA_ELF;
		strcpy( d.name, "mJabber" );	
		strcpy( d.elf_path, "/b/Elf2/mJabber/mJabber.elf" );

		List_AppendItem( &ditems, &d );
		//---------------------------
		d.x = 34;
		d.y = 35;
		d.event = 982;

		d.dtype = DE_DATA_EVENT;
		strcpy( d.name, "java" );

		List_AppendItem( &ditems, &d );
		//---------------------------
		d.x = 66;
		d.y = 35;
		d.event = 0;

		d.dtype = DE_DATA_ELF;
		strcpy( d.name, "MotoCmd2" );	
		strcpy( d.elf_path, "/b/Elf2/MotoCmd2.elf" );


		List_AppendItem( &ditems, &d );
		//---------------------------
		d.x = 100;
		d.y = 35;
		d.event = 948;

		d.dtype = DE_DATA_EVENT;
		strcpy( d.name, "bluetooth" );	

		List_AppendItem( &ditems, &d );
		//---------------------------
		d.x = 134;
		d.y = 35;
		d.event = 1022;

		d.dtype = DE_DATA_EVENT;
		strcpy( d.name, "usb" );	

		List_AppendItem( &ditems, &d );
		SaveSetting();
	}
	
	return RESULT_OK;
}

// записываем в конфиг
UINT32 SaveSetting()
{
	FILE_HANDLE_T 			f;

	
	// запись в файл
	f = DL_FsOpenFile(file_cfg, FILE_WRITE_MODE, 0);
	
	List_WriteFile( &ditems, f );
		
	DL_FsCloseFile(f);
	
	dbg("Save Setting");
	return RESULT_OK;
}

DESKTOP_ITEM * ElfGetDItem( int i )
{

	return (DESKTOP_ITEM *)List_GetItem(&ditems, i);
}

UIX_ICON_T ElfGetIcon( int i )
{
	UIX_ICON_T		icon;
	
		icon.bmp = NULL;
		icon.point.x = i * 16;
		icon.point.y = 0;
	
	return icon;
}
UINT32 InitResources()
{
	WCHAR			icon_path[256], buf[32];
	int				i;
	
	dbgf("InitResources: enter");
	
	for ( i = 0; i < List_GetNum(&ditems); i++ ) {
	
		memclr( &ElfGetDItem( i )->bg, sizeof(AHIBITMAP_T) );
		memclr( &ElfGetDItem( i )->bitmap, sizeof(AHIBITMAP_T) );
		
		u_strcpy( icon_path, icons_dir );
		u_strcat( icon_path, u_atou(ElfGetDItem( i )->name, buf) );
		u_strcat( icon_path, L".bmp" );
		
		if( !DL_FsFFileExist(icon_path) ) 
			u_mkpath( icon_path, icons_dir, L"default.bmp", 0 );
			
		BMP_LoadFromFile( icon_path, &ElfGetDItem( i )->bitmap );

		ElfGetDItem( i )->bg.width = ElfGetDItem( i )->bitmap.width;
		ElfGetDItem( i )->bg.height = ElfGetDItem( i )->bitmap.height;
		ElfGetDItem( i )->bg.format = AHIFMT_16BPP_565;
		ElfGetDItem( i )->bg.stride = (ElfGetDItem( i )->bg.width * 2 + 3) & (~3);
		ElfGetDItem( i )->bg.image = malloc( ElfGetDItem( i )->bg.stride * ElfGetDItem( i )->bg.height );

	}
	
	u_mkpath( icon_path, icons_dir, L"cursor.bmp", 0 );
	memclr( &cusor_bitmap, sizeof(AHIBITMAP_T) );
	BMP_LoadFromFile( icon_path, &cusor_bitmap );
	
	dbgf("InitResources: done");
		
	return RESULT_OK;
}

UINT32 FreeResources()
{
	int				i;
	
	dbgf("FreeResources: enter");
		
		for ( i = 0; i < List_GetNum(&ditems); i++ ) {

		if ( ElfGetDItem( i )->bg.image )  mfree( ElfGetDItem( i )->bg.image ); ElfGetDItem( i )->bg.image = NULL;
		if ( ElfGetDItem( i )->bitmap.image ) mfree( ElfGetDItem( i )->bitmap.image ); ElfGetDItem( i )->bitmap.image = NULL;
	}
	if ( cusor_bitmap.image ) mfree( cusor_bitmap.image ); cusor_bitmap.image = NULL;
	
	dbgf("FreeResources: done");
	
	return RESULT_OK;
}

#ifdef	DEBUG
void ReportPhoneInfo()
{
	dbg("");
	
	dbgf("PhoneInfo: PhoneName = %s", ldrGetPhoneName());
	dbgf("PhoneInfo: PlatformName = %s", ldrGetPlatformName());
	dbgf("PhoneInfo: FirmwareVersion = %s", ldrGetFirmwareVersion());
	
	dbg("");
	const AHIDRVINFO_T	*ahi = ldrGetAhiDrvInfo();
	dbgf("PhoneInfo: ahi.drvName = %s", ahi->drvName);
	dbgf("PhoneInfo: ahi.drvVer = %s", ahi->drvVer);
	dbgf("PhoneInfo: ahi.totalMemory = %d b", ahi->totalMemory);
	dbgf("PhoneInfo: ahi.internalMemSize = %d b", ahi->internalMemSize);
	dbgf("PhoneInfo: ahi.externalMemSize = %d b", ahi->externalMemSize);
	dbg("");
}
#endif
